/*******************************************************************************
*    Comm2J 1.0.                                                               *
*    Copyright (C) 2004 Luca Bertoncello                                       *
*    Hartigstrae, 12 - 01127 Dresden Deutschland                              *
*    E-Mail: lucabert@lucabert.com, lucabert@lucabert.de                       *
*    http://www.lucabert.com/  http://www.lucabert.de/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

package comm2j;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

/**
 * MessageI is the basic interface used to send and receive message
 * to/from remote server
 * @see Comm2J
 * @author Luca Bertoncello &lt;lucabert@lucabert.de&gt;
 * @version 1.0
*/

public interface MessageI
{
  /**
   * Sets the message
   * @param message Message to be used
   */
  public void setMessage(Object message);

  /**
   * Returns the current message
   * @return The current message
   */
  public Object getMessage();

  /**
   * Reads a message from give InputStream.
   * @param is The InputStream used to read data
   * @return Number of read bytes
   * @throws java.io.IOException if an error occours
   */
  public int readMessage(InputStream is) throws IOException;

  /**
   * Send current message throw given OutputStream
   * @param os The OutputStream used to read data
   * @return Number of wrote bytes
   * @throws java.io.IOException if an error occours
   */
  public int writeMessage(OutputStream os) throws IOException;

  /**
   * Used to know if a message was set (by setMessage or writeMessage)
   * @return true is message was set
   */
  public boolean isMessageSet();

  /**
   * Send an header identifying the message type
   * throw the give OutputStream
   * @param os The OutputStream used to read data
   * @return Number of wrote bytes
   * @throws java.io.IOException if an error occours
   */
  public int writeHeader(OutputStream os) throws IOException;

  /**
   * Gets the header identifying the message type
   * from the give InputStream
   * @param is The InputStream used to read data
   * @return The message type
   * @throws java.io.IOException if an error occours
   */
  public String readHeader(InputStream is) throws IOException;

  /**
   * Returns a string with the content of the message
   * @return A String identifying the message
   */
  public String toString();

  /**
   * Returns the name of the message class
   * @return The class name
   */
  public String getName();
}
